@@ -29,7 +29,7 @@ int pretty = A2T_PRETTY | A2T_NAME;
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s [ -b ] [ -d ] [ -n ] \n",name);
+    fprintf(stderr,"usage: %s [ -b ] [ -d ] [ -l logfile ] [ -n ] \n",name);
     exit(1);
 }
 
@@ -50,6 +50,9 @@ int main(int argc,char **argv)
 		set_verbosity(NULL,DIAG_DEBUG);
 		debug = 1;
 		break;
+	    case 'l':
+		set_logfile(optarg);
+		break;
 	    case 'n':
 		pretty = A2T_PRETTY;
 		break;
