@@ -511,8 +511,9 @@ void need_ip(int itf_num,unsigned long ip)
 	diag(COMPONENT,DIAG_ERROR,"itf %d not found",itf_num);
 	return;
     }
-    if (!itf->arp_srv) return; /* bad luck - no ARP server */
     entry = lookup_ip(itf,ip);
+    if ((!entry || entry->state != as_valid) && !itf->arp_srv)
+	return; /* bad luck - no ARP server when we need one */
     if (entry)
 	switch (entry->state) {
 	    case as_resolv:
