@@ -84,7 +84,6 @@ void ilmi_loop(int fd, int itf)
 	{
 	case RESTART:
 	  deleteNetPrefix();
-	  delete_nsap(itf);
 	  diag(COMPONENT, DIAG_DEBUG, "sending cold-start");
 	  diag(COMPONENT, DIAG_DEBUG, "sending get-next");
 	  send_message(fd, coldstart_message);
@@ -154,7 +153,7 @@ void ilmi_loop(int fd, int itf)
 			    if(in_message->data->a.get_response->error_status == NOERROR)
 			      {
 				diag(COMPONENT, DIAG_INFO, "ATM address registered");
-				add_nsap(itf, netprefix_oid, esi_oid);
+				update_nsap(itf, netprefix_oid, esi_oid);
 				poll_misses = 0;
 				new_state = POLL;
 			      }
