@@ -85,27 +85,10 @@ AsnOid *get_esi(int fd)
   return name;
 }
 
-void delete_nsap(int itf)
+void update_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
 {
   struct atmif_sioc req;
-  int fd;
-  
-  if ((fd = socket(AF_ATMSVC, SOCK_DGRAM, 0)) < 0)
-    diag(COMPONENT, DIAG_FATAL, "socket: %s", strerror(errno));
-
-  req.number = itf;
-  req.arg = NULL;
-
-  if(ioctl(fd, ATM_RSTADDR, &req) < 0)
-    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_RSTADDR: %s", strerror(errno));
-
-  close(fd);
-}
-
-void add_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
-{
-  struct atmif_sioc req;
-  struct sockaddr_atmsvc addr;
+  struct sockaddr_atmsvc addr, ouraddr;
   int fd, m, n;
 
   addr.sas_family = AF_ATMSVC;
@@ -129,14 +112,41 @@ void add_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
 
   if ((fd = socket(AF_ATMSVC, SOCK_DGRAM, 0)) < 0)
     diag(COMPONENT, DIAG_FATAL, "socket: %s", strerror(errno));
-
+  
   req.number = itf;
-  req.arg = &addr;
-  req.length = sizeof(addr);
+  req.arg = &ouraddr;
+  req.length = sizeof(ouraddr);
 
-  if(ioctl(fd, ATM_ADDADDR, &req) < 0)
-    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_ADDADDR: %s", strerror(errno));
+  /* Try to get our address on that interface */
 
+  if (ioctl(fd, ATM_GETADDR, &req) <0)
+    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_GETADDR: %s", strerror(errno));
+
+  n = 1;
+
+  for(m = 0; m < netprefix->octetLen; m++)
+     if(addr.sas_addr.prv[m] != ouraddr.sas_addr.prv[m])
+     {	n = 0;
+	break;
+     }
+
+  if ((req.length == 0) || (n == 0)) {
+  	  req.number = itf;
+  	  req.arg = NULL;
+  	  req.length = 0;
+
+  	  if(ioctl(fd, ATM_RSTADDR, &req) < 0)
+    	    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_RSTADDR: %s", strerror(errno));
+
+	  req.number = itf;
+	  req.arg = &addr;
+	  req.length = sizeof(addr);
+	  
+	  if(ioctl(fd, ATM_ADDADDR, &req) < 0)
+	    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_ADDADDR: %s", strerror(errno));
+
+	  diag(COMPONENT, DIAG_INFO, "ATM NSAP Address set on local host");
+  }	     	
   close(fd);
 
 }
