@@ -28,7 +28,7 @@ int main(int argc,char **argv)
     struct sockaddr_atmpvc addr;
     struct atm_qos qos;
     const char *name;
-    int s,null;
+    int s,null,itf;
 
     name = argv[0];
     null = 0;
@@ -44,10 +44,6 @@ int main(int argc,char **argv)
 	perror("socket");
 	return 1;
     }
-    if (ioctl(s,CLIP_PVC,0) < 0) {
-	perror("ioctl CLIP_PVC");
-	return 1;
-    }
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
 	usage(argv[0]);
@@ -68,10 +64,15 @@ int main(int argc,char **argv)
 	perror("bind");
 	return 1;
     }
+    if ((itf = ioctl(s,CLIP_PVC,0)) < 0) {
+	perror("ioctl CLIP_PVC");
+	return 1;
+    }
     if (null)
 	if (ioctl(s,CLIP_NULENCAP,0) < 0) {
 	    perror("ioctl CLIP_NULENCAP");
 	    return 1;
 	}
+    printf("atm%d\n",itf);
     return 0;
 }
