@@ -26,7 +26,7 @@
  *
  * Client registration/query funcs used in Digital's code
  *
- * $Id: address.c,v 1.12 1996/07/07 11:51:47 carnil Exp carnil $
+ * $Id: address.c,v 1.13 1996/08/06 14:14:11 carnil Exp carnil $
  *
  */
 
@@ -55,6 +55,7 @@
 #include "codes.h"
 #include "g_event.h"
 #include "utl_os.h"
+#include "le_disp.h"
 
 #define EMOD MOD_ADDR_REG
 #define EINST "address.c"
@@ -71,6 +72,7 @@ typedef struct _client_t_ {
   ADDR_ATM addr; /* ATM address for this client */  
   struct _client_t_ *next; 
   HANDLE my_timer; /* See that ATM address is maintained */
+  int notify;
 } Client_t;
 
 Client_t *Clientlist = NULL;
@@ -112,6 +114,7 @@ addr_reg_client_register (HANDLE addr_reg_handle, ADDR_REG_CALLBACK callback,
     }
     to_register->name[0] = '\0';
   }
+  to_register->notify=0;
   *p_client_handle = to_register;
   memset(&to_register->addr,0,sizeof(ADDR_ATM));
   to_register->next = Clientlist;
@@ -120,6 +123,21 @@ addr_reg_client_register (HANDLE addr_reg_handle, ADDR_REG_CALLBACK callback,
 			&to_register->my_timer);
 }
 
+int 
+addr_reg_client_deregister(HANDLE client_handle)
+{
+  Client_t *client = client_handle;
+
+  if (!client) {
+    EVENT(EM_NERR,("Address registration deregistering nonexisting client\n"));
+    return -1;
+  }
+  os_timer_dealloc(client->my_timer);
+  mem_free(EINST, client->name);
+  mem_free(EINST, client);
+  return 0;
+}
+
 STATUS 
 addr_reg_atm_addr_alloc(HANDLE client_handle)
 {
@@ -129,6 +147,7 @@ addr_reg_atm_addr_alloc(HANDLE client_handle)
 
   client = (Client_t *)client_handle;
 
+  client->notify=1;
   addr_reg_timer_callback((HANDLE)client);
   return STATUS_K_SUCCESS;
 }
@@ -148,6 +167,14 @@ addr_reg_atm_addr_dealloc (HANDLE client_handle, ADDR_ATM *p_atm_addr)
   return STATUS_K_SUCCESS;
 }
 
+void
+addr_set_atm_addr(ADDR_ATM *set_to)
+{
+  Client_t *client = Clientlist;
+
+  ATM_COPY(*set_to, client->addr);
+}
+
 int
 address_convert(char* parsestring, ADDR_ATM *atm_addr)
 {
@@ -184,92 +211,129 @@ void
 addr_reg_timer_callback(HANDLE context)
 {
   Client_t *client = (Client_t *)context;
-  struct sockaddr_atmsvc ouraddr;
+  unsigned char *buffer;
+  struct sockaddr_atmsvc *ouraddr;
   struct atmif_sioc req;
-  int s;
+  int s,i;
+  static int address_invalidated =0; /* If address is missing twice,
+					then act */
 
   EVENT(EM_DEBUG,("Addr_reg_timer_called\n"));
+  buffer = (unsigned char*)mem_alloc(EINST, 5*sizeof(struct sockaddr_atmsvc));
   req.number = 0;
-  req.arg = &ouraddr;
-  req.length = sizeof(ouraddr);
+  req.arg = buffer;
+  req.length = 5*sizeof(*ouraddr);
 
   /* Reset (or set) timer */
-  os_timer_set(client->my_timer, ADDR_TIMER_FREQ);
+  os_timer_set(client->my_timer, ADDR_TIMER_FREQ+1);
 
   s = socket(PF_ATMSVC, SOCK_DGRAM, 0);
   if (s<0) {
     EVENT(EM_NERR,("Socket failed:%s\n",strerror(errno)));
+    mem_free(EINST, buffer);
     return;
   }
   if (ioctl(s, ATM_GETADDR, &req) <0) {
     close(s);
     EVENT(EM_NERR,("Ioctl failed:%s\n",strerror(errno)));
+    mem_free(EINST, buffer);
     return;
   }
   close(s);
 
   if (req.length ==0 ) { /* No ATM address */
-    if (ATM_EQUAL_NULL(client->addr)) /* Address hasn't been registered 
-				       * and wasn't registered now */
+    if (ATM_EQUAL_NULL(client->addr)) {/* Address hasn't been registered 
+				        * and wasn't registered now */
+      mem_free(EINST, buffer);
       return;
-    EVENT(EM_NERR,("Address invalidated\n"));
-    client->callback(client->callback_handle, &client->addr,
-		     ADDR_REG_EVENT_ATM_ADDR_INVALIDATED);
-    ATM_COPY_NULL(client->addr);
-    return;
+    }
+    if (address_invalidated) { /* Twice is enough */
+      EVENT(EM_NERR,("Address invalidated\n"));
+      client->callback(client->callback_handle, &client->addr,
+		       ADDR_REG_EVENT_ATM_ADDR_INVALIDATED);
+      ATM_COPY_NULL(client->addr);
+      mem_free(EINST, buffer);
+      return;
+    } else {
+      address_invalidated = 1;
+      mem_free(EINST, buffer);
+      return;
+    }
   }
-
+  address_invalidated = 0;
   EVENT(EM_DEBUG,("We have address\n"));
-  if (!ATM_EQUAL_NULL(client->addr)){  /* Address has already been registered*/
-    if (client->addr.prefix[0] == ouraddr.sas_addr.prv[0] &&
-	client->addr.prefix[1] == ouraddr.sas_addr.prv[1] &&
-	client->addr.prefix[2] == ouraddr.sas_addr.prv[2] &&
-	client->addr.prefix[3] == ouraddr.sas_addr.prv[3] &&
-	client->addr.prefix[4] == ouraddr.sas_addr.prv[4] &&
-	client->addr.prefix[5] == ouraddr.sas_addr.prv[5] &&
-	client->addr.prefix[6] == ouraddr.sas_addr.prv[6] &&
-	client->addr.prefix[7] == ouraddr.sas_addr.prv[7] &&
-	client->addr.prefix[8] == ouraddr.sas_addr.prv[8] &&
-	client->addr.prefix[9] == ouraddr.sas_addr.prv[9] &&
-	client->addr.prefix[10] == ouraddr.sas_addr.prv[10] &&
-	client->addr.prefix[11] == ouraddr.sas_addr.prv[11] &&
-	client->addr.prefix[12] == ouraddr.sas_addr.prv[12] &&
-	client->addr.esi[0] == ouraddr.sas_addr.prv[13] &&
-	client->addr.esi[1] == ouraddr.sas_addr.prv[14] &&
-	client->addr.esi[2] == ouraddr.sas_addr.prv[15] &&
-	client->addr.esi[3] == ouraddr.sas_addr.prv[16] &&
-	client->addr.esi[4] == ouraddr.sas_addr.prv[17] &&
-	client->addr.esi[5] == ouraddr.sas_addr.prv[18] &&
-	client->addr.sel == ouraddr.sas_addr.prv[19])/*...and has stayed same*/
-      return;
+  if (!client->notify && !ATM_EQUAL_NULL(client->addr)){  /* Address has already been registered*/
+    EVENT(EM_DEBUG,("Old %s\n",disp_atm_text(client->addr)));
+    for(i=0;i < req.length/sizeof(struct sockaddr_atmsvc); i++) {
+      /* Possibility for several ATM addresses */   
+      ouraddr = (struct sockaddr_atmsvc*)buffer+i;
+      EVENT(EM_DEBUG,("Tested agaist %2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x\n",
+		      ouraddr->sas_addr.prv[0],ouraddr->sas_addr.prv[1],
+		      ouraddr->sas_addr.prv[2],ouraddr->sas_addr.prv[3],
+		      ouraddr->sas_addr.prv[4],ouraddr->sas_addr.prv[5],
+		      ouraddr->sas_addr.prv[6],ouraddr->sas_addr.prv[7],
+		      ouraddr->sas_addr.prv[8],ouraddr->sas_addr.prv[9],
+		      ouraddr->sas_addr.prv[10],ouraddr->sas_addr.prv[11],
+		      ouraddr->sas_addr.prv[12],ouraddr->sas_addr.prv[13],
+		      ouraddr->sas_addr.prv[14],ouraddr->sas_addr.prv[15],
+		      ouraddr->sas_addr.prv[16],ouraddr->sas_addr.prv[17],
+		      ouraddr->sas_addr.prv[18],ouraddr->sas_addr.prv[19]));
+      if (client->addr.prefix[0] == ouraddr->sas_addr.prv[0] &&
+	  client->addr.prefix[1] == ouraddr->sas_addr.prv[1] &&
+	  client->addr.prefix[2] == ouraddr->sas_addr.prv[2] &&
+	  client->addr.prefix[3] == ouraddr->sas_addr.prv[3] &&
+	  client->addr.prefix[4] == ouraddr->sas_addr.prv[4] &&
+	  client->addr.prefix[5] == ouraddr->sas_addr.prv[5] &&
+	  client->addr.prefix[6] == ouraddr->sas_addr.prv[6] &&
+	  client->addr.prefix[7] == ouraddr->sas_addr.prv[7] &&
+	  client->addr.prefix[8] == ouraddr->sas_addr.prv[8] &&
+	  client->addr.prefix[9] == ouraddr->sas_addr.prv[9] &&
+	  client->addr.prefix[10] == ouraddr->sas_addr.prv[10] &&
+	  client->addr.prefix[11] == ouraddr->sas_addr.prv[11] &&
+	  client->addr.prefix[12] == ouraddr->sas_addr.prv[12] &&
+	  client->addr.esi[0] == ouraddr->sas_addr.prv[13] &&
+	  client->addr.esi[1] == ouraddr->sas_addr.prv[14] &&
+	  client->addr.esi[2] == ouraddr->sas_addr.prv[15] &&
+	  client->addr.esi[3] == ouraddr->sas_addr.prv[16] &&
+	  client->addr.esi[4] == ouraddr->sas_addr.prv[17] &&
+	  client->addr.esi[5] == ouraddr->sas_addr.prv[18] &&
+	  client->addr.sel == ouraddr->sas_addr.prv[19]) {
+	/*...and has stayed same*/
+	mem_free(EINST, buffer);
+	return;
+      }
+    }
     EVENT(EM_DEBUG,("... which is new one\n"));
     /* Hasn't stayed same. First invalidate previous, then register new. */
     client->callback(client->callback_handle, &client->addr,
 		     ADDR_REG_EVENT_ATM_ADDR_INVALIDATED);
   }
   EVENT(EM_DEBUG,("Register new one\n"));
-  client->addr.prefix[0] = ouraddr.sas_addr.prv[0];
-  client->addr.prefix[1] = ouraddr.sas_addr.prv[1];
-  client->addr.prefix[2] = ouraddr.sas_addr.prv[2];
-  client->addr.prefix[3] = ouraddr.sas_addr.prv[3];
-  client->addr.prefix[4] = ouraddr.sas_addr.prv[4];
-  client->addr.prefix[5] = ouraddr.sas_addr.prv[5];
-  client->addr.prefix[6] = ouraddr.sas_addr.prv[6];
-  client->addr.prefix[7] = ouraddr.sas_addr.prv[7];
-  client->addr.prefix[8] = ouraddr.sas_addr.prv[8];
-  client->addr.prefix[9] = ouraddr.sas_addr.prv[9];
-  client->addr.prefix[10] = ouraddr.sas_addr.prv[10];
-  client->addr.prefix[11] = ouraddr.sas_addr.prv[11];
-  client->addr.prefix[12] = ouraddr.sas_addr.prv[12];
-  client->addr.esi[0] = ouraddr.sas_addr.prv[13];
-  client->addr.esi[1] = ouraddr.sas_addr.prv[14];
-  client->addr.esi[2] = ouraddr.sas_addr.prv[15];
-  client->addr.esi[3] = ouraddr.sas_addr.prv[16];
-  client->addr.esi[4] = ouraddr.sas_addr.prv[17];
-  client->addr.esi[5] = ouraddr.sas_addr.prv[18];
-  client->addr.sel = ouraddr.sas_addr.prv[19];
+  client->notify=0;
+  ouraddr = (struct sockaddr_atmsvc *)buffer;
+  client->addr.prefix[0] = ouraddr->sas_addr.prv[0];
+  client->addr.prefix[1] = ouraddr->sas_addr.prv[1];
+  client->addr.prefix[2] = ouraddr->sas_addr.prv[2];
+  client->addr.prefix[3] = ouraddr->sas_addr.prv[3];
+  client->addr.prefix[4] = ouraddr->sas_addr.prv[4];
+  client->addr.prefix[5] = ouraddr->sas_addr.prv[5];
+  client->addr.prefix[6] = ouraddr->sas_addr.prv[6];
+  client->addr.prefix[7] = ouraddr->sas_addr.prv[7];
+  client->addr.prefix[8] = ouraddr->sas_addr.prv[8];
+  client->addr.prefix[9] = ouraddr->sas_addr.prv[9];
+  client->addr.prefix[10] = ouraddr->sas_addr.prv[10];
+  client->addr.prefix[11] = ouraddr->sas_addr.prv[11];
+  client->addr.prefix[12] = ouraddr->sas_addr.prv[12];
+  client->addr.esi[0] = ouraddr->sas_addr.prv[13];
+  client->addr.esi[1] = ouraddr->sas_addr.prv[14];
+  client->addr.esi[2] = ouraddr->sas_addr.prv[15];
+  client->addr.esi[3] = ouraddr->sas_addr.prv[16];
+  client->addr.esi[4] = ouraddr->sas_addr.prv[17];
+  client->addr.esi[5] = ouraddr->sas_addr.prv[18];
+  client->addr.sel = ouraddr->sas_addr.prv[19];
   client->callback(client->callback_handle,&client->addr, 
 		   ADDR_REG_EVENT_ATM_ADDR_ALLOCATED);
+  mem_free(EINST, buffer);
 }
 
 int 
@@ -292,9 +356,57 @@ addr_getesi(unsigned char *mac_addr)
   }
   return 0;
 }
+
+int
+addr_getouratmaddr(struct sockaddr_atmsvc *addr)
+{
+  Client_t *client;
+
+  client = Clientlist;
+
+  if (!client) {
+    EVENT(EM_SERR,("Addr_getouratmaddr, no ATM address!!!\n"));
+    return -1;
+  }
+  if (ATM_EQUAL_NULL(client->addr)) {
+    addr_reg_timer_callback(client);
+  }
+  if (ATM_EQUAL_NULL(client->addr)) {
+    EVENT(EM_SERR,("Addr_getouratmaddr, no ATM address!!!\n"));
+    return -1;
+  }
+  memset(addr,0,sizeof(*addr));
+  addr->sas_family = AF_ATMSVC;
+  addr->sas_addr.prv[0] = client->addr.prefix[0];
+  addr->sas_addr.prv[1] = client->addr.prefix[1];
+  addr->sas_addr.prv[2] = client->addr.prefix[2];
+  addr->sas_addr.prv[3] = client->addr.prefix[3];
+  addr->sas_addr.prv[4] = client->addr.prefix[4];
+  addr->sas_addr.prv[5] = client->addr.prefix[5];
+  addr->sas_addr.prv[6] = client->addr.prefix[6];
+  addr->sas_addr.prv[7] = client->addr.prefix[7];
+  addr->sas_addr.prv[8] = client->addr.prefix[8];
+  addr->sas_addr.prv[9] = client->addr.prefix[9];
+  addr->sas_addr.prv[10] = client->addr.prefix[10];
+  addr->sas_addr.prv[11] = client->addr.prefix[11];
+  addr->sas_addr.prv[12] = client->addr.prefix[12];
+  addr->sas_addr.prv[13] = client->addr.esi[0];
+  addr->sas_addr.prv[14] = client->addr.esi[1];
+  addr->sas_addr.prv[15] = client->addr.esi[2];
+  addr->sas_addr.prv[16] = client->addr.esi[3];
+  addr->sas_addr.prv[17] = client->addr.esi[4];
+  addr->sas_addr.prv[18] = client->addr.esi[5];
+  addr->sas_addr.prv[19] = client->addr.sel;
+  return 0;
+}
+
+
 /*
  *
  * $Log: address.c,v $
+ * Revision 1.13  1996/08/06 14:14:11  carnil
+ * Address handling moved to address.c from conn.c
+ *
  * Revision 1.12  1996/07/07 11:51:47  carnil
  * Address changes are noticed and notified
  *
