@@ -25,7 +25,7 @@
  *
  * Connection forming/teardown & packet send/receive
  *
- * $Id: conn.c,v 1.25 1996/07/07 11:51:47 carnil Exp carnil $
+ * $Id: conn.c,v 1.26 1996/08/06 14:14:11 carnil Exp carnil $
  *
  */
 
@@ -41,6 +41,7 @@
 #include <errno.h>
 #include <assert.h>
 #include <netinet/in.h>
+#include <fcntl.h>
 
 /* Atm includes */
 
@@ -52,8 +53,6 @@
 #include "g_types.h"
 #include "cm_sap.h"
 #include "lec.h"
-#include "map.h"
-#include "map_data.h"
 #include "atm.h"
 #include "cm.h"
 #include "g_event.h"
@@ -63,6 +62,7 @@
 #include "le_disp.h"
 #include "kernel_itf.h"
 #include "lec_ctrl.h"
+#include "addr_reg.h"
 
 #define EMOD MOD_CM
 #define EINST "conn.c"
@@ -81,6 +81,7 @@
 			   */
 #define TRANSMITTED 1247  /* Packet transmitted, not notified */
 #define RELEASED 1248     /* Connection released */
+#define CONNECTING 1249   /* Non blocking socket, connecting */
 
 /* type */
 #define WEMADE 7
@@ -132,7 +133,6 @@ static int list_add_conn(HANDLE sap_handle, HANDLE conn_context,
 			 long age_limit, CONN_INFO *info, Conn_t **p_conn);
 static Conn_t* conn_already_exists(unsigned char *atm_addr,Conn_t *current);
 
-
 STATUS 
 cm_sap_register(HANDLE sap_context, 
 		SAP_RCV_CALLBACK rcv_callback, 
@@ -194,7 +194,7 @@ cm_sap_unregister(HANDLE sap_handle)
 {
   Sap_client_t *sap, *curr;
   Sap_client_t *prev = NULL;
-  Conn_t *conn;
+  Conn_t *conn, *next;
 
   sap = (Sap_client_t *)sap_handle;
 
@@ -216,29 +216,29 @@ cm_sap_unregister(HANDLE sap_handle)
     prev->next = curr->next;    
   }
   /* Remove vcc's */
-  for(conn= connlist;conn!=NULL;conn=conn->next) {
+  for(conn= connlist;conn;conn=next) {
+    EVENT(EM_DEBUG,("Destroying:%p fd:%d type:%d\n",conn,conn->fd,conn->type));
+    next=conn->next;
     if (conn->type == KERNEL_SOCK) {
+      EVENT(EM_DEBUG,("Closing kernel socket\n"));
       close(conn->fd);
       list_remove_conn(conn);
+      mem_free(EINST, conn);
     }
   }
-  for(conn=connlist;conn!=NULL;conn=conn->next) {
+  for(conn=connlist;conn;conn=next) {
+    next=conn->next;
     close(conn->fd);
+    if (conn->conn_info)
+      mem_free(EINST, conn->conn_info);
+    close(conn->fd);
+    list_remove_conn(conn);
+    mem_free(EINST, conn);
   }
   mem_free(EINST, sap->p_sap_text);
   mem_free(EINST, sap);
 }
 
-void 
-cm_map_get(LINK_CALLBACK link_update_callback,
-	   LINK_CALLBACK link_remove_callback, HANDLE callback_context,
-	   HANDLE *p_vc_list, HANDLE *p_atm_list)
-{
-  EVENT(EM_ASSERT,("Cm_map_get\n"));
-  *p_vc_list = NULL;
-  *p_atm_list=NULL;
-}
-
 /* Don't check parameter current for atm address match */
 static Conn_t*
 conn_already_exists(unsigned char *atm_addr, Conn_t *current)
@@ -319,17 +319,19 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
       /* Try to remove possible entry in kernel */
       lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
     }
+    *p_conn_handle = NULL;
     return STATUS_K_RESOURCES;
   }
   /* Initialize address structs */
   memset(&us,0,sizeof(us));
-  if (get_ouratmaddr(s,&us) <0) {
+  if (addr_getouratmaddr(&us) <0) {
     if (is_data_direct(p_conn_info)) {
       /* Try to remove possible entry in kernel */
       lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
     }
 
     close(s);
+    *p_conn_handle = NULL;
     return STATUS_K_ATM_RESOURCES;
   }
   
@@ -346,6 +348,7 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
       lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
     }
     close(s);
+    *p_conn_handle = NULL;
     return STATUS_K_ATM_RESOURCES;
   }
   
@@ -362,6 +365,7 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
     }
 
     close(s);
+    *p_conn_handle = NULL;
     return STATUS_K_ATM_RESOURCES;
   }
 
@@ -391,15 +395,34 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
 		  p_conn_info->addr.sas_addr.prv[19]&0xff));
 
   /* Try to connect */
+  if (is_data_direct(p_conn_info)) {
+    ret = fcntl(s, F_GETFL);
+    if (ret <0) {
+      EVENT(EM_SERR, ("fcntl(s, F_GETFL\n"));
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+      close(s);
+      *p_conn_handle = NULL;
+    }
+    if (fcntl(s, F_SETFL, ret|O_NONBLOCK)<0) {
+      EVENT(EM_SERR, ("fcntl(s, F_SETFL, x&O_NONBLOCK)\n"));
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+      close(s);
+      *p_conn_handle=NULL;
+      return STATUS_K_ATM_RESOURCES;
+    }
+  }
   ret = connect(s, (struct sockaddr *)&(p_conn_info->addr), 
 		sizeof(struct sockaddr_atmsvc));
-  if (ret<0) {
-    EVENT(EM_NERR,("connect error:%s\n",strerror(errno)));
+  if (ret<0 && errno != EINPROGRESS) {
+    EVENT(EM_NERR,("connect error:%d,%s\n",errno,strerror(errno)));
     if (is_data_direct(p_conn_info)) {
       /* Try to remove possible entry in kernel */
       lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
     }
+    if (errno == ENETRESET)
+      sig_reset(0);    
     close(s);
+    *p_conn_handle = NULL;
     return STATUS_K_ATM_RESOURCES;
   }
 
@@ -410,16 +433,19 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   case 0x03:
     memcpy(ioc_data.atm_addr, p_conn_info->addr.sas_addr.prv, ATM_ESA_LEN);
     ioc_data.receive = 0;
+    EVENT(EM_DEBUG,("About to ioctl(s, ATMLEC_DATA, &ioc_data)\n"));
     if (ioctl(s, ATMLEC_DATA, &ioc_data)<0) {
       EVENT(EM_SERR,("Can't change socket into LE data socket:%s\n",
-		     strerror(errno)));
+		     strerror(errno)));      
       if (is_data_direct(p_conn_info)) {
 	/* Try to remove possible entry in kernel */
       lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
       }
       close(s);
+      *p_conn_handle = NULL;
       return STATUS_K_ATM_RESOURCES;
     }
+    EVENT(EM_DEBUG,("ioctl done\n"));
     break;
   case 0x04:
   case 0x05:
@@ -431,6 +457,7 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
 	lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
       }
       close(s);
+      *p_conn_handle = NULL;
       return STATUS_K_ATM_RESOURCES;
     }
     break;
@@ -448,13 +475,17 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
       lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
     }
     close(s);
+    *p_conn_handle = NULL;
     return STATUS_K_RESOURCES;
   }
   conn->fd = s;
   conn->type = WEMADE;
 
   /* Connection was made. Must notify upper level later. */
-  conn->status = CONNECTED;
+  if (is_data_direct(p_conn_info))
+    conn->status = CONNECTING;
+  else
+    conn->status = CONNECTED;
 
   EVENT(EM_DEBUG,("Conn_handle %lx,fd:%d\n",(long)conn, s));
   *p_conn_handle = conn;
@@ -462,19 +493,20 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
 }
 
 STATUS 
-cm_sap_svc_teardown(HANDLE conn_handle,
-		    RELEASE_INFO *p_release_info)
+cm_sap_svc_teardown(HANDLE conn_handle)
 {
   Conn_t *conn;
 
   EVENT(EM_DEBUG,("Cm_sap_svc_teardown %lx\n",(long)conn_handle));
   conn = (Conn_t *)conn_handle;
 
-  if (conn->conn_info) {
+  close(conn->fd);
+  /* Remove conn from list */
+  list_remove_conn(conn);
+  if (conn->conn_info)
     mem_free(EINST, conn->conn_info);
-    conn->conn_info = NULL;
-  }
-  conn->status = RELEASED;
+  mem_free(EINST,conn);
+
   return STATUS_K_SUCCESS;
 }
 
@@ -487,11 +519,10 @@ cm_sap_xmt_vc (HANDLE conn_handle, void *p_data,
   Sap_client_t *sap;
   int ret;
 
-  EVENT(EM_DEBUG,("Cm_sap_xmt_vc\n"));
-
   /* Write data out */
   conn = (Conn_t *)conn_handle;
   sap = (Sap_client_t *)conn->sap_handle;
+  EVENT(EM_DEBUG,("Cm_sap_xmt_vc fd:%d len:%ld\n",conn->fd, length));
   ret=write(conn->fd, p_data, length);
 
   /* Tried to transmit packet */
@@ -532,13 +563,6 @@ svcinit_conn_info(CONN_INFO *p_svc_conn_info)
   p_svc_conn_info->addr.sas_addr.blli = &(p_svc_conn_info->blli);
 }
 
-void 
-svc_release_info_make(int code, RELEASE_INFO *p_release_info)
-{
-  assert(p_release_info);
-}
-
-
 /* Callbacks for connections */
 void
 conn_call_callbacks(void)
@@ -580,6 +604,8 @@ conn_call_callbacks(void)
       close(conn->fd);
       /* Remove conn from list */
       list_remove_conn(conn);
+      if (conn->conn_info)
+	mem_free(EINST, conn->conn_info);
       mem_free(EINST,conn);
       conn=c_tmp;
     } else {
@@ -617,20 +643,51 @@ conn_get_fds(fd_set *fds)
   conn = connlist;
 
   if (EMASK & EM_DEBUG) {
-    buff = (char*)malloc(1024);
+    buff = (char*)mem_alloc(EINST,1024);
     ch = buff;
   }
   while (conn) {
-    if (EMASK & EM_DEBUG) {
-      ch = ch+sprintf(ch,"-%3d %s",conn->fd, conn_get_type_string(conn->type));
+    if (conn->status != CONNECTING) {
+      if (EMASK & EM_DEBUG) {
+	ch=ch+sprintf(ch,"-%3d %s",conn->fd, conn_get_type_string(conn->type));
+      }
+      FD_SET(conn->fd, fds);
     }
-    FD_SET(conn->fd, fds);
     conn = conn->next;
   }
   EVENT(EM_DEBUG,("conn_add_fds:%s\n",buff));
+  if (EMASK & EM_DEBUG) 
+    mem_free(EINST, buff);
   fflush(stdout);
 }
 
+void
+conn_get_connecting_fds(fd_set *fds)
+{
+  Conn_t *conn;
+  char *buff, *ch;
+
+  conn = connlist;
+
+  if (EMASK & EM_DEBUG) {
+    buff = (char*)mem_alloc(EINST, 1024);
+    ch = buff;
+  }
+  while (conn) {
+    if (conn->status == CONNECTING) {
+      if (EMASK & EM_DEBUG) {
+	ch=ch+sprintf(ch,"-%3d %s",conn->fd, conn_get_type_string(conn->type));
+      }
+      FD_SET(conn->fd, fds);
+    }
+    conn = conn->next;
+  }
+  EVENT(EM_DEBUG,("conn_add_connecting_fds:%s\n",buff));
+  if (EMASK & EM_DEBUG) 
+    mem_free(EINST, buff);
+  fflush(stdout);
+}
+  
 int
 conn_check_incoming(fd_set *fds)
 {
@@ -644,38 +701,43 @@ conn_check_incoming(fd_set *fds)
   struct sockaddr_atmsvc addr;
   Conn_t *to_add;
   struct atmlec_ioc ioc_data;
+  int reset = 0;
 
   conn = connlist;
-  
+
   while (conn) {
-    if (FD_ISSET(conn->fd, fds)) {
-      EVENT(EM_DEBUG,("Event in fd:%d\n",conn->fd));
-      if (conn->type == KERNEL_SOCK) { /* Message from kernel */
-	kernel_dispatch_handlers();
-      } else if (conn->type == LISTENING) { /* Incoming call */
-	EVENT(EM_EVENT,("Incoming call\n"));
-	if (list_add_conn(conn->sap_handle,
-			  NULL,
-			  0,
-			  NULL,
-			  &to_add)<0) {
-	  EVENT(EM_NERR,("list_conn_add failed\n"));
-	  continue;
-	}	       
-	sap = (Sap_client_t *)conn->sap_handle;
-	if (sap->svc_connect_callback(conn->conn_context,
-				      conn->conn_info, 
-				      to_add,
-				      &to_add->conn_context,
-				      &to_add->age_limit,
-				      &reject_reason) == FALSE) {
-	  /* What to do, if ... */
-	  assert(0);
-	}
+    if (FD_ISSET(conn->fd, fds) && conn->type == LISTENING){/* Incoming call */
+      EVENT(EM_EVENT,("Incoming call\n"));
+      if (list_add_conn(conn->sap_handle,
+			NULL,
+			0,
+			NULL,
+			&to_add)<0) {
+	EVENT(EM_NERR,("list_conn_add failed\n"));
+	continue;
+      }	       
+      sap = (Sap_client_t *)conn->sap_handle;
+      if (sap->svc_connect_callback(conn->conn_context,
+				    conn->conn_info, 
+				    to_add,
+				    &to_add->conn_context,
+				    &to_add->age_limit,
+				    &reject_reason) == FALSE) {
+	/* What to do, if ... */
 	len = sizeof(addr);
 	fd = accept(conn->fd, (struct sockaddr *)&addr, &len);
-	if (fd<0) {	  
+	if (fd <0 && (errno == EUNATCH || errno == ENETRESET))
+	  sig_reset(42);
+	if (fd >=0)
+	  close(fd);
+	list_remove_conn(to_add);
+      } else {
+	len = sizeof(addr);
+	fd = accept(conn->fd, (struct sockaddr *)&addr, &len);
+	if (fd<0) {
 	  EVENT(EM_NERR,("Accept failed:%s\n",strerror(errno)));
+	  if (errno == ENETRESET || errno == EUNATCH)
+	    sig_reset(42);
 	  list_remove_conn(to_add);
 	  sap->vc_notify_callback(to_add->conn_context,
 				  to_add,
@@ -731,7 +793,7 @@ conn_check_incoming(fd_set *fds)
 		  conn->conn_info->blli.l3.tr9577.snap[4] == 3) {
 		tmpconn = conn_already_exists(addr.sas_addr.prv, to_add);
 		if (tmpconn) {
-		  get_ouratmaddr(conn->fd, &addr);
+		  addr_getouratmaddr(&addr);
 		  if (memcmp(addr.sas_addr.prv, ioc_data.atm_addr, ATM_ESA_LEN)
 		      <0)
 		    ioc_data.receive = 1;
@@ -742,26 +804,46 @@ conn_check_incoming(fd_set *fds)
 	      } else
 		ioc_data.receive = 2; /* Multicast distribute */
 	      
+	      EVENT(EM_DEBUG,("About to ioctl(fd, ATMLEC_DATA, &ioc_data)\n"));
 	      if (ioctl(fd, ATMLEC_DATA, &ioc_data)<0) {
 		EVENT(EM_SERR,("Can't change socket into LE data socket:%s\n",
 			       strerror(errno)));
 	      }
+	      EVENT(EM_DEBUG,("ioctl done\n"));
 	    }
 	  }
 	}
+      }
+    }
+    conn = conn->next;
+  }
+  conn=connlist;
+  while (conn) {
+    if (FD_ISSET(conn->fd, fds)) {
+      EVENT(EM_DEBUG,("Event in fd:%d\n",conn->fd));
+      if (conn->type == KERNEL_SOCK) { /* Message from kernel */
+	kernel_dispatch_handlers();
+      } else if (conn->type == LISTENING) { /* Incoming call */
+      } else if (conn->status == CONNECTING) { /* Call is ready */
+	memset(&addr,0,sizeof(addr));
+	if (!connect(conn->fd, (struct sockaddr*)&addr,sizeof(addr))) {
+	  EVENT(EM_EVENT,("Nonblocking connect() :%s\n",strerror(errno)));
+	}
+	conn->status = CONNECTED;
       } else { /* Data coming in */
 	nbytes = read(conn->fd, buffer, BUFSIZE);
 	if (nbytes <0) {
 	  EVENT(EM_SERR,("Read failed on fd %d: %s\n",
 			 conn->fd,strerror(errno)));
-	  if (errno == EUNATCH || errno == ENOENT) 
-	    return -1;
+	  if (errno == EUNATCH || errno == ENOENT || errno == ENETRESET) 
+	    reset = -1;
 	  else {
 	    conn->status = RELEASED;
 	    EVENT(EM_EVENT, ("Closing connection\n"));
 	  }
 	} else if (nbytes==0) {
 	  /* EOF, connection closed */
+	  
 	  conn->status = RELEASED;
 	  EVENT(EM_EVENT,("Connection closed\n"));
 	} else {
@@ -782,7 +864,7 @@ conn_check_incoming(fd_set *fds)
     }
     conn = conn->next;
   }
-  return 0;
+  return reset;
 }
 
 int
@@ -834,7 +916,10 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
     return -1;
   }
   
-  conn_info->addr.sas_family = AF_ATMSVC;
+  if (addr_getouratmaddr(&conn_info->addr)<0) {
+    close(fd);
+    return -1;
+  }
   conn_info->addr.sas_addr.blli = &conn_info->blli;
   conn_info->blli.l3_proto = ATM_L3_TR9577;
   conn_info->blli.l3.tr9577.ipi = NLPID_IEEE802_1_SNAP;
@@ -846,6 +931,10 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
   conn_info->blli.l3.tr9577.snap[4] = (unsigned char)(0xff&blli_codepoint);
     
   len = sizeof(struct sockaddr_atmsvc);  
+
+  if (EMASK & EM_DEBUG)
+    disp_sockaddr(&conn_info->addr);
+  
   ret = bind(fd, (struct sockaddr *)&conn_info->addr, len);
   if (ret != 0) {
     EVENT(EM_NERR,("Bind failed: %s\n",strerror(errno)));
@@ -877,19 +966,6 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
   return fd;
 }
 
-void 
-cm_reset(HANDLE sap_handle)
-{
-  Conn_t *conn, *next;
-
-  /* Remove vcc's */
-  for(conn=connlist;conn!=NULL;conn=next) {
-    next = conn->next;
-    close(conn->fd);
-    list_remove_conn(conn);
-  }
-}
-
 static int
 list_add_conn(HANDLE sap_handle, HANDLE conn_context, long age_limit, 
 	      CONN_INFO *info, Conn_t **p_conn)
@@ -923,13 +999,18 @@ list_remove_conn(HANDLE conn_handle)
 
   conn = (Conn_t *)conn_handle;
 
-  EVENT(EM_DEBUG,("Removing conn:%p previous:%p next:%p ", conn,
+  if (conn->next == NULL && conn->previous == NULL
+      && connlist != conn) return;
+  EVENT(EM_DEBUG,("Removing conn:%p fd:%d previous:%p next:%p ", conn, 
+		  conn->fd,
 		  conn->previous, conn->next));
   if (conn->previous) 
-    EVENT(EM_DEBUG,("Previous:%p, next:%p, previous:%p ", conn->previous,
+    EVENT(EM_DEBUG,("Previous:%p, fd:%d, next:%p, previous:%p ", conn->previous,
+		    conn->previous->fd,
 		    conn->previous->next, conn->previous->previous));
   if (conn->next)
-    EVENT(EM_DEBUG,("Next:%p, next:%p, previous:%p ",conn->next,
+    EVENT(EM_DEBUG,("Next:%p, fd:%d next:%p, previous:%p ",conn->next,
+		    conn->next->fd,
 		    conn->next->next, conn->next->previous));  
   if (conn->previous) {
     conn->previous->next = conn->next;
@@ -956,25 +1037,14 @@ conn_set_kernel_socket(int fd)
   return fd;
 }
 
-int
-get_ouratmaddr(int s, struct sockaddr_atmsvc *addr)
-{
-  struct atmif_sioc req;
-
-  req.number = 0;
-  req.arg = addr;
-  req.length = sizeof(struct sockaddr_atmsvc);
-
-  if (ioctl(s, ATM_GETADDR, &req) <0) {
-    EVENT(EM_SERR,("ioctl ATM_GETADDR :%s",strerror(errno)));
-    return -1;
-  }
-  return 0;
-}
 
 /*
  *
  * $Log: conn.c,v $
+ * Revision 1.26  1996/08/06 14:14:11  carnil
+ * Cleaning up
+ * Address handling moved to address.c from conn.c
+ *
  * Revision 1.25  1996/07/07 11:51:47  carnil
  * Global msg mask
  *
