@@ -78,7 +78,6 @@
 *       - lc_join_start
 *         - (ELAN_STATUS_CALLBACK)
 *           - lc_addr_resolve
-*       - lc_event_mask_set
 *       - lc_deregister
 *     - lc_destroy
 *
@@ -311,83 +310,6 @@ STATUS lc_register (HANDLE       lc_handle,
 --*/
 void lc_deregister (HANDLE lc_elan_handle);
 
-
-/*++
-* =====================
-* = lc_event_mask_set =
-* =====================
-*
-* Overview:
-*
-* Arguments:
-*   lc_elan_handle  - (IN) Handle returned by lc_register.
-*   event_mask      - (IN) New value of log event mask.
-*
-* Returns:
-*   None
-*
-* Preconditions:
-*   None
-*
-* Postconditions:
-*   None
---*/
-void lc_event_mask_set (HANDLE lc_elan_handle,
-                        UINT32 event_mask);
-
-
-/*++
-* ===================
-* = lc_mac_addr_add =
-* ===================
-*
-* Overview:
-*   Registers a new destination address with the LES.  Note that MAC address
-*   registration is asynchronous and does not take effect immediately.  This
-*   routine begins the registration process.  There is no notification to the
-*   caller that registration is complete.  It simply takes effect when the
-*   registration process completes.
-*
-* Arguments:
-*   lc_elan_handle  - (IN) Handle returned by lc_register.
-*   mac_addr        - (IN) New LAN destination for this elan instance.
-*
-* Returns:
-*   STATUS_K_SUCCESS - Successful Completion (registration is in process)
-*
-* Preconditions:
-*   None
-*
-* Postconditions:
-*   None
---*/
-STATUS lc_mac_addr_add (HANDLE lc_elan_handle,
-                        ESI    mac_addr);
-
-/*++
-* ===================
-* = lc_mac_addr_del =
-* ===================
-*
-* Overview:
-*   Deregisters a registered destination address with the LES.
-*
-* Arguments:
-*   lc_elan_handle  - (IN) Handle returned by lc_register.
-*   mac_addr        - (IN) LAN destination to be deregistered.
-*
-* Returns:
-*   None
-*
-* Preconditions:
-*   None
-*
-* Postconditions:
-*   None
---*/
-void lc_mac_addr_del (HANDLE lc_elan_handle,
-                      ESI    mac_addr);
-
 /*++
 * =========================
 * = lc_dest_is_registered =
@@ -540,7 +462,6 @@ void lc_sap_rcv (HANDLE     conn_context,
 void lc_sap_link_status (HANDLE          lc_elan_handle,
                          ATM_LINK_STATE  link_state);
 
-
 /* 
  * Forwards address deletion request to lec_arp module.
  */
