@@ -9,6 +9,7 @@
 #include <sys/socket.h>
 #include <linux/atm.h>
 
+#define HOSTS_ATM "/etc/hosts.atm"
 
 #define T2A_PVC		 1	/* address is PVC */
 #define T2A_SVC		 2	/* address is SVC */
@@ -26,6 +27,8 @@
 
 #define MAX_ATM_ADDR_LEN (2*ATM_ESA_LEN+ATM_E164_LEN+5)
 				/* 4 dots, 1 plus */
+#define MAX_ATM_NAME_LEN 256 /* wild guess */
+
 
 int text2atm(const char *text,struct sockaddr *addr,int length,int flags);
 int atm2text(char *buffer,int length,const struct sockaddr *addr,int flags);
