@@ -5,6 +5,11 @@
 
 #define MAX_Q_MSG 500
 
+/* Coding Standards */
+
+#define Q2931_CS_ITU		0	/* ITU-T standardized */
+#define Q2931_CS_NET		3	/* Standard defined for the network */
+
 /* Message types */
 
 #define QMSG_SETUP		0x05
