@@ -3,10 +3,14 @@
 /* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
-int q_dump = 0;
-
-
 #ifdef DUMP_MODE
+static int q_dump = 0;
+#else
+int q_dump = 0;
+#endif
+
+
+#ifdef STANDALONE
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -100,26 +104,6 @@ static unsigned long q_get(unsigned char *table,int pos,int size)
 }
 
 
-#if 0
-static void q_put(unsigned char *table,int pos,int size,unsigned long value)
-{
-    unsigned char *byte;
- 
-    byte = &table[pos >>3];
-    if (pos & 7) {
-	*byte++ |= value << (pos & 7);
-	value >>= 8-(pos & 7);
-	size -= 8-(pos & 7);
-    }
-    while (size > 0) {
-	size -= 8;
-	*byte++ |= value;
-	value >>= 8;
-    }
-}
-#endif
- 
-
 static void q_copy(unsigned char *src,int pos,unsigned char *dst,int size)
 {
     src += pos >> 3;
@@ -139,7 +123,7 @@ static void q_copy(unsigned char *src,int pos,unsigned char *dst,int size)
 }
 
 
-void q_start(void)
+void PREFIX(start)(void)
 {
     q_init_global();
 }
@@ -189,6 +173,8 @@ static int q_init(Q_DSC *dsc)
 }
 
 
+#ifndef DUMP_MODE
+
 static void use_group(Q_DSC *dsc,int group)
 {
     int *scan;
@@ -311,6 +297,8 @@ int q_read(Q_DSC *dsc,int field,void *buf,int size)
     return len;
 }
 
+#endif
+
 
 #define LENGTH_STACK 10
 
@@ -606,7 +594,7 @@ static int q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 }
 
 
-int q_open(Q_DSC *dsc,void *buf,int size)
+int PREFIX(open)(Q_DSC *dsc,void *buf,int size)
 {
     dsc->buffer = NULL;
     q_init(dsc);
@@ -614,6 +602,8 @@ int q_open(Q_DSC *dsc,void *buf,int size)
 }
 
 
+#ifndef DUMP_MODE
+
 int q_create(Q_DSC *dsc,void *buf,int size)
 {
    dsc->buffer = buf;
@@ -622,8 +612,10 @@ int q_create(Q_DSC *dsc,void *buf,int size)
    return 0;
 }
 
+#endif
 
-int q_close(Q_DSC *dsc)
+
+int PREFIX(close)(Q_DSC *dsc)
 {
     int size;
 
@@ -638,7 +630,7 @@ int q_close(Q_DSC *dsc)
 }
 
 
-#ifdef DUMP_MODE
+#ifdef STANDALONE
 
 int main(int argc,const char **argv)
 {
@@ -649,7 +641,9 @@ int main(int argc,const char **argv)
     debug = argc != 1;
     len = 0;
     while (scanf("%x",&c) == 1) msg[len++] = c;
-    q_open(&dsc,msg,len);
+    qd_start();
+    qd_open(&dsc,msg,len);
+    qd_close(&dsc);
     return 0;
 }
 
