@@ -5,7 +5,13 @@
  
 #ifndef QLIB_H
 #define QLIB_H
- 
+
+#ifdef DUMP_MODE
+#define PREFIX(x) qd_##x
+#else
+#define PREFIX(x) q_##x
+#endif
+
 
 #include "q.out.h"
 
@@ -36,10 +42,10 @@ typedef struct {
 } Q_DSC;
  
  
-void q_start(void);
-int q_open(Q_DSC *dsc,void *buf,int size);
+void PREFIX(start)(void);
+int PREFIX(open)(Q_DSC *dsc,void *buf,int size);
 int q_create(Q_DSC *dsc,void *buf,int size);
-int q_close(Q_DSC *dsc);
+int PREFIX(close)(Q_DSC *dsc);
 
 void q_assign(Q_DSC *dsc,int field,unsigned long value);
 void q_write(Q_DSC *dsc,int field,const void *buf,int size);
