@@ -19,6 +19,7 @@
 #include "io.h"
 #include "proto.h"
 #include "saal.h"
+#include "trace.h"
 
 
 #define COMPONENT "SIGD"
@@ -31,6 +32,7 @@ extern FILE *yyin;
 int net = 0;
 int debug = 0;
 int pretty = A2T_PRETTY | A2T_NAME;
+int sig_pcr = -1;
 
 static SAAL_DSC saal;
 
@@ -38,7 +40,7 @@ static SAAL_DSC saal;
 static void usage(const char *name)
 {
     fprintf(stderr,"usage: %s [ -b ] [ -c config_file ] [ -d ] [ -l logfile ] "
-      "[ -n ] [ -N ]\n",name);
+      "[ -n ] [ -N ] [ -P pcr ]\n",name);
     exit(1);
 }
 
@@ -71,12 +73,14 @@ void from_net(void *buffer,int size)
 
 void to_signaling(void *msg,int size)
 {
+    trace_q2931("TO NETWORK",msg,size);
     saal_send(&saal,msg,size);
 }
 
 
 static void q_data_ind(void *user_data,void *data,int length)
 {
+    trace_q2931("FROM NETWORK",data,length);
     to_q2931(data,length);
 }
 
@@ -213,6 +217,7 @@ static void setup_signals(void)
 int main(int argc,char **argv)
 {
     const char *config_file;
+    char *end;
     int c,background;
 
     set_application("atmsigd");
@@ -220,7 +225,7 @@ int main(int argc,char **argv)
     background = 0;
     memset(&signaling_pvc,0,sizeof(signaling_pvc));
     signaling_pvc.sap_addr.vci = 5;
-    while ((c = getopt(argc,argv,"bc:dl:nN")) != EOF)
+    while ((c = getopt(argc,argv,"bc:dl:nNP:")) != EOF)
 	switch (c) {
 	    case 'b':
 		background = 1;
@@ -244,6 +249,10 @@ int main(int argc,char **argv)
 	    case 'N':
 		net = 1;
 		break;
+	    case 'P':
+		sig_pcr = strtol(optarg,&end,0);
+		if (*end) usage(argv[0]);
+		break;
 	    default:
 		usage(argv[0]);
 	}
