@@ -21,6 +21,7 @@
 
 #include "proto.h"
 #include "io.h"
+#include "trace.h"
 
 
 #define COMPONENT "IO"
@@ -73,6 +74,7 @@ static void recv_kernel(void)
     if (size == KERNEL_BUFFER_SIZE) /* gotcha ! */
 	diag(COMPONENT,DIAG_FATAL,"kernel message too big (>= %d)",
 	  KERNEL_BUFFER_SIZE);
+    trace_kernel("FROM KERNEL",(struct atmsvc_msg *) buffer,size);
     bllis = (size-KERNEL_BASE_LEN)/sizeof(struct atm_blli);
     if (size != KERNEL_BASE_LEN+bllis*sizeof(struct atm_blli))
 	diag(COMPONENT,DIAG_FATAL,
@@ -93,6 +95,7 @@ void to_kernel(struct atmsvc_msg *msg)
     diag("KERNEL",DIAG_DEBUG,"TO KERNEL: %s (%d) for 0x%lx/0x%lx <%d>",
       as_name[msg->type],msg->reply,msg->vcc,msg->listen_vcc,size);
 	/* should be "IO" ... */
+    trace_kernel("TO KERNEL",msg,size);
     wrote = write(kernel,msg,size);
     if (wrote == size) return;
     if (wrote < 0) {
@@ -123,7 +126,12 @@ static int open_signaling(void)
 	return -1;
     }
     memset(&qos,0,sizeof(qos));
-    qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_UBR;
+    if (sig_pcr == -1)
+	qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_UBR;
+    else {
+	qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_CBR;
+	qos.rxtp.min_pcr = qos.txtp.min_pcr = sig_pcr;
+    }
     qos.rxtp.max_sdu = qos.txtp.max_sdu = MAX_Q_MSG;
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
 	perror("setsockopt SO_ATMQOS");
