@@ -154,7 +154,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 
 		for (walk = local_addr; walk->state != ls_unused; walk++)
 		    if (walk->state == ls_same && atm_equal(&walk->addr,
-		      &msg->svc,0,0)) break;
+		      &msg->svc,(ATM_ESA_LEN-1)*8,AXE_WILDCARD)) break;
 		if (walk->state == ls_unused)
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
 		else send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,NULL,NULL);
