@@ -32,8 +32,8 @@ const char *qs_name[] = {
 	"CONN_REQ",	"IN_PROC",	"ACTIVE",	"REL_REQ",
 	"REL_IND" };
 
-const char *as_name[] = { "<invalid>","as_bind","as_establish","as_listen",
-  "as_okay","as_indicate","as_close" };
+const char *as_name[] = { "<invalid>","as_bind","as_connect","as_accept",
+  "as_listen","as_okay","as_error","as_indicate","as_close" };
 
 const Q2931_STATE state_map[] = { /* formatting aligned with STATE */
 	qs_null,	qs_null,	qs_null,	qs_call_init,
@@ -58,7 +58,7 @@ SOCKET *new_sock(unsigned long id)
     sock = alloc_t(SOCKET);
     sock->state = ss_invalid;
     sock->pvc.sap_addr.vpi = sock->pvc.sap_addr.vci = 0;
-    sock->pvc.sap_txtp.class = sock->pvc.sap_rxtp.class = ATM_UBR;
+    sock->qos.txtp.traffic_class = sock->qos.rxtp.traffic_class = ATM_UBR;
     sock->id = id;
     sock->local = sock->remote = NULL;
     sock->error = 0;
@@ -107,14 +107,16 @@ void new_state(SOCKET *sock,STATE state)
 }
 
 
-SOCKET *lookup_sap(struct sockaddr_atmsvc *sap,struct sockaddr_atmsvc **res)
+SOCKET *lookup_sap(const struct sockaddr_atmsvc *sap,const struct atm_qos *qos,
+    struct sockaddr_atmsvc **res_sap,struct atm_qos *res_qos)
 {
     SOCKET *walk,*wildcard;
 
     wildcard = NULL;
     for (walk = sockets; walk; walk = walk->next)
 	if (walk->state == ss_listening) {
-	    if (walk->local && sap_compat(walk->local,sap,res)) return walk;
+	    if (walk->local && sap_compat(walk->local,sap,res_sap,&walk->qos,
+	      qos,res_qos)) return walk;
 	    if (!walk->local) wildcard = walk;
 	}
     return wildcard;
@@ -161,8 +163,9 @@ const char *mid2name(unsigned char mid)
 
 
 void send_kernel(unsigned long vcc,unsigned long listen_vcc,
-  enum atmsvc_msg_type type,int reply,struct sockaddr_atmpvc *pvc,
-  struct sockaddr_atmsvc *svc,struct sockaddr_atmsvc *local)
+  enum atmsvc_msg_type type,int reply,const struct sockaddr_atmpvc *pvc,
+  const struct sockaddr_atmsvc *svc,const struct sockaddr_atmsvc *local,
+  const struct atm_qos *qos)
 {
     struct atmsvc_msg *msg;
     struct atm_blli *walk;
@@ -178,6 +181,8 @@ void send_kernel(unsigned long vcc,unsigned long listen_vcc,
     msg->reply = reply;
     if (pvc) msg->pvc = *pvc;
     else memset(&msg->pvc,0,sizeof(msg->pvc));
+    if (qos) msg->qos = *qos;
+    else memset(&msg->qos,0,sizeof(msg->qos));
     if (!local) memset(&msg->local,0,sizeof(msg->local));
     else {
 	msg->local = *local;
@@ -243,7 +248,7 @@ void set_error(SOCKET *sock,int code)
 void send_close(SOCKET *sock)
 {
     if (sock->error == 1234) diag(COMPONENT,DIAG_ERROR,"BUG! BUG! BUG!");
-    SEND_ERROR(sock->id,sock->error);
+    send_kernel(sock->id,0L,as_close,sock->error,NULL,NULL,NULL,NULL);
     sock->error = 1234;
 }
 
