@@ -378,7 +378,8 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    {
 		unsigned char cause;
 
-		if (mid != QMSG_REL_COMP) cause = 0;
+		if (mid != QMSG_REL_COMP || !q_present(&in_dsc,QF_cause))
+		    cause = 0;
 		else {
 		    cause = q_fetch(&in_dsc,QF_cause);
 		    diag(COMPONENT,DIAG_DEBUG,"Cause %d (%s)",cause,
