@@ -61,12 +61,14 @@ struct sockaddr_atmsvc *sap_copy(const struct sockaddr_atmsvc *sap)
 static int class_compat(const struct atm_trafprm *tx,
   const struct atm_trafprm *rx)
 {
-    if (tx->class == ATM_NONE || tx->class == ATM_ANYCLASS) return 1;
-    if (rx->class == ATM_UBR || rx->class == ATM_ANYCLASS) return 1;
+    if (tx->traffic_class == ATM_NONE || tx->traffic_class == ATM_ANYCLASS)
+	return 1;
+    if (rx->traffic_class == ATM_UBR || rx->traffic_class == ATM_ANYCLASS)
+	return 1;
 	/* don't apply CAC to PCR */
-    if (tx->class != rx->class) return 0;
+    if (tx->traffic_class != rx->traffic_class) return 0;
 	/* ignore special cases like CBR to VBR for now */
-    switch (tx->class) {
+    switch (tx->traffic_class) {
 	case ATM_CBR:
 	    if (!rx->max_pcr || rx->max_pcr == ATM_MAX_PCR) return 1;
 	    return tx->min_pcr <= rx->max_pcr;
@@ -74,7 +76,7 @@ static int class_compat(const struct atm_trafprm *tx,
 	       bandwidth negotiation anyway. */
 	default:
 	    diag(COMPONENT,DIAG_ERROR,"unsupported traffic class %d\n",
-	      tx->class);
+	      tx->traffic_class);
 	    return 0;
     }
 }
@@ -151,47 +153,56 @@ static int blli_compat(const struct atm_blli *old,const struct atm_blli *new,
 }
 
 
-static int do_sap_compat(const struct sockaddr_atmsvc *old,
-  const struct sockaddr_atmsvc *new,struct sockaddr_atmsvc *res)
+static int do_sap_compat(const struct sockaddr_atmsvc *old_sap,
+  const struct sockaddr_atmsvc *new_sap,const struct atm_qos *old_qos,
+  const struct atm_qos *new_qos,struct sockaddr_atmsvc *res_sap,
+  struct atm_qos *res_qos)
 {
-    if (old->sas_txtp.max_sdu && new->sas_rxtp.max_sdu &&
-      old->sas_txtp.max_sdu > new->sas_rxtp.max_sdu) return 0;
-    if (new->sas_txtp.max_sdu && old->sas_rxtp.max_sdu &&
-      new->sas_txtp.max_sdu > old->sas_rxtp.max_sdu) return 0;
-    if (!class_compat(&old->sas_txtp,&new->sas_rxtp) ||
-      !class_compat(&new->sas_txtp,&old->sas_rxtp)) return 0;
-    if (!bhli_compat(&old->sas_addr.bhli,&new->sas_addr.bhli,
-      res ? &res->sas_addr.bhli : NULL)) return 0;
-    return blli_compat(old->sas_addr.blli,new->sas_addr.blli,
-      res ? &res->sas_addr.blli : NULL,res ? (struct atm_blli *) (res+1) :
-      NULL);
+    if (res_qos) *res_qos = *new_qos;
+    if (old_qos->txtp.max_sdu && new_qos->rxtp.max_sdu &&
+      old_qos->txtp.max_sdu > new_qos->rxtp.max_sdu) return 0;
+    if (new_qos->txtp.max_sdu && old_qos->rxtp.max_sdu &&
+      new_qos->txtp.max_sdu > old_qos->rxtp.max_sdu) return 0;
+    if (!class_compat(&old_qos->txtp,&new_qos->rxtp) ||
+      !class_compat(&new_qos->txtp,&old_qos->rxtp)) return 0;
+    if (res_sap) *res_sap = *new_sap;
+    if (!bhli_compat(&old_sap->sas_addr.bhli,&new_sap->sas_addr.bhli,
+      res_sap ? &res_sap->sas_addr.bhli : NULL)) return 0;
+    return blli_compat(old_sap->sas_addr.blli,new_sap->sas_addr.blli,
+      res_sap ? &res_sap->sas_addr.blli : NULL,
+      res_sap ? (struct atm_blli *) (res_sap+1) : NULL);
 }
 
 
-int sap_compat(const struct sockaddr_atmsvc *old,
-  const struct sockaddr_atmsvc *new,struct sockaddr_atmsvc **out)
+int sap_compat(const struct sockaddr_atmsvc *old_sap,
+  const struct sockaddr_atmsvc *new_sap,struct sockaddr_atmsvc **out_sap,
+  const struct atm_qos *old_qos,const struct atm_qos *new_qos,
+  struct atm_qos *out_qos)
 {
-    struct sockaddr_atmsvc *res;
+    struct sockaddr_atmsvc *res_sap;
+    struct atm_qos res_qos;
     int compat;
 
-    if ((*old->sas_addr.prv || *old->sas_addr.pub) && !atm_equal(old,new,0,0))
-	return 0;
-    if (!out) res = NULL;
-    else res = alloc(sizeof(struct sockaddr_atmsvc)+sizeof(struct atm_blli));
-    compat = do_sap_compat(old,new,res);
-    if (out)
-	if (!compat) free(res);
+    if ((*old_sap->sas_addr.prv || *old_sap->sas_addr.pub) &&
+      !atm_equal(old_sap,new_sap,0,0)) return 0;
+    if (!out_sap) res_sap = NULL;
+    else res_sap = alloc(sizeof(struct sockaddr_atmsvc)+
+	  sizeof(struct atm_blli));
+    compat = do_sap_compat(old_sap,new_sap,old_qos,new_qos,res_sap,&res_qos);
+    if (out_sap)
+	if (!compat) free(res_sap);
 	else {
-	    *out = res;
-	    *res = *new;
-	    if (res->sas_addr.blli)
-		res->sas_addr.blli = (struct atm_blli *) (res+1);
+	    *out_sap = res_sap;
+	    if (res_sap->sas_addr.blli)
+		res_sap->sas_addr.blli = (struct atm_blli *) (res_sap+1);
 	}
+    if (out_qos && compat) *out_qos = res_qos;
     return compat;
 }
 
 
-int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
+int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap,
+  const struct atm_qos *qos)
 {
     struct atm_blli *blli;
     int pcr;
@@ -199,9 +210,10 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
     if (!*sap->sas_addr.prv) return -EDESTADDRREQ;
     else q_write(dsc,QF_cdpn_esa,(void *) sap->sas_addr.prv,ATM_ESA_LEN);
 	/* improve @@@ */
-    switch (sap->sas_txtp.class) {
+    switch (qos->txtp.traffic_class) {
 	case ATM_NONE:
-	    if (sap->sas_rxtp.class == ATM_CBR) q_assign(dsc,QF_fw_pcr_01,0);
+	    if (qos->rxtp.traffic_class == ATM_CBR)
+		q_assign(dsc,QF_fw_pcr_01,0);
 	    break;
 	case ATM_UBR:
 	    diag(COMPONENT,DIAG_DEBUG,"UBR");
@@ -211,33 +223,35 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
 	    break;
 	case ATM_CBR:
 	    /* here's a bit of policy: send the highest value we have */
-	    pcr = SELECT_TOP_PCR(sap->sas_txtp);
+	    pcr = SELECT_TOP_PCR(qos->txtp);
 	    diag(COMPONENT,DIAG_DEBUG,"CBR fwd %d (%d..%d)",pcr,
-	      sap->sas_txtp.min_pcr,sap->sas_txtp.max_pcr);
+	      qos->txtp.min_pcr,qos->txtp.max_pcr);
 	    q_assign(dsc,QF_fw_pcr_01,pcr);
 	    break;
         default:
-	    diag(COMPONENT,DIAG_ERROR,"bad TX class (%d)",sap->sas_txtp.class);
+	    diag(COMPONENT,DIAG_ERROR,"bad TX class (%d)",
+	      qos->txtp.traffic_class);
 	    return -EINVAL;
     }
-    switch (sap->sas_txtp.class == ATM_UBR ? ATM_NONE : sap->sas_rxtp.class) {
+    switch (qos->txtp.traffic_class == ATM_UBR ? ATM_NONE :
+      qos->rxtp.traffic_class) {
 	case ATM_NONE:
-	    if (sap->sas_txtp.class == ATM_CBR) q_assign(dsc,QF_bw_pcr_01,0);
+	    if (qos->txtp.traffic_class == ATM_CBR)
+		q_assign(dsc,QF_bw_pcr_01,0);
 	    break;
 	case ATM_CBR:
-	    pcr = SELECT_TOP_PCR(sap->sas_rxtp);
+	    pcr = SELECT_TOP_PCR(qos->rxtp);
 	    diag(COMPONENT,DIAG_DEBUG,"CBR bwd %d (%d..%d)",pcr,
-	      sap->sas_rxtp.min_pcr,sap->sas_rxtp.max_pcr);
+	      qos->rxtp.min_pcr,qos->rxtp.max_pcr);
 	    q_assign(dsc,QF_bw_pcr_01,pcr);
 	    break;
         default:
-	    diag(COMPONENT,DIAG_ERROR,"bad RX class (%d)",sap->sas_txtp.class);
+	    diag(COMPONENT,DIAG_ERROR,"bad RX class (%d)",
+	      qos->rxtp.traffic_class);
 	    return -EINVAL;
     }
-    if (sap->sas_txtp.max_sdu)
-	q_assign(dsc,QF_fw_max_sdu,sap->sas_txtp.max_sdu);
-    if (sap->sas_rxtp.max_sdu)
-	q_assign(dsc,QF_bw_max_sdu,sap->sas_rxtp.max_sdu);
+    if (qos->txtp.max_sdu) q_assign(dsc,QF_fw_max_sdu,qos->txtp.max_sdu);
+    if (qos->rxtp.max_sdu) q_assign(dsc,QF_bw_max_sdu,qos->rxtp.max_sdu);
     /* @@@ bearer class ? */
     /* @@@ QOS class ? */
     if (sap->sas_addr.bhli.hl_type != ATM_HL_NONE) {
@@ -325,7 +339,7 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
 }
 
 
-struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc)
+struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc,struct atm_qos *qos)
 {
     struct sockaddr_atmsvc *sap;
     struct atm_blli *blli;
@@ -333,6 +347,7 @@ struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc)
     sap = alloc(sizeof(struct sockaddr_atmsvc)+sizeof(struct atm_blli));
     blli = (struct atm_blli *) (sap+1);
     memset(sap,0,sizeof(struct sockaddr_atmsvc)+sizeof(struct atm_blli));
+    memset(qos,0,sizeof(struct atm_qos));
     sap->sas_family = AF_ATMSVC;
     if (q_present(dsc,QF_cdpn_esa)) /* @@@ handle E.164 too */
 	(void) q_read(dsc,QF_cdpn_esa,(void *) &sap->sas_addr.prv,ATM_ESA_LEN);
@@ -341,32 +356,32 @@ struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc)
 	    diag(COMPONENT,DIAG_ERROR,"AAL type %d requested",
 	      q_fetch(dsc,QF_aal_type));
     if (q_present(dsc,QF_best_effort)) {
-	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_UBR;
+	qos->txtp.traffic_class = qos->rxtp.traffic_class = ATM_UBR;
 	diag(COMPONENT,DIAG_DEBUG,"UBR");
     }
     else {
-	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_CBR;
-	sap->sas_txtp.max_pcr = sap->sas_rxtp.max_pcr = 0;
+	qos->txtp.traffic_class = qos->rxtp.traffic_class = ATM_CBR;
+	qos->txtp.max_pcr = qos->rxtp.max_pcr = 0;
 	/* unbalanced decoding - always sets upper bound */
 	if (q_present(dsc,QF_fw_pcr_01)) {
-	    sap->sas_rxtp.min_pcr = 0;
-	    sap->sas_rxtp.max_pcr = q_fetch(dsc,QF_fw_pcr_01);
+	    qos->rxtp.min_pcr = 0;
+	    qos->rxtp.max_pcr = q_fetch(dsc,QF_fw_pcr_01);
 	}
 	if (q_present(dsc,QF_bw_pcr_01)) {
-	    sap->sas_txtp.min_pcr = 0;
-	    sap->sas_txtp.max_pcr = q_fetch(dsc,QF_bw_pcr_01);
+	    qos->txtp.min_pcr = 0;
+	    qos->txtp.max_pcr = q_fetch(dsc,QF_bw_pcr_01);
 	}
-	if (!sap->sas_txtp.max_pcr) sap->sas_txtp.class = ATM_NONE;
-	if (!sap->sas_rxtp.max_pcr) sap->sas_rxtp.class = ATM_NONE;
+	if (!qos->txtp.max_pcr) qos->txtp.traffic_class = ATM_NONE;
+	if (!qos->rxtp.max_pcr) qos->rxtp.traffic_class = ATM_NONE;
 	diag(COMPONENT,DIAG_DEBUG,"CBR fwd %d..%d bwd %d..%d",
-	  sap->sas_rxtp.min_pcr,sap->sas_rxtp.max_pcr,sap->sas_txtp.min_pcr,
-	  sap->sas_txtp.max_pcr);
+	  qos->rxtp.min_pcr,qos->rxtp.max_pcr,qos->txtp.min_pcr,
+	  qos->txtp.max_pcr);
 	/* SHOULD ... fail call if anything is missing ... @@@ */
     }
     if (q_present(dsc,QF_bw_max_sdu))
-	sap->sas_txtp.max_sdu = q_fetch(dsc,QF_bw_max_sdu);
+	qos->txtp.max_sdu = q_fetch(dsc,QF_bw_max_sdu);
     if (q_present(dsc,QF_fw_max_sdu))
-	sap->sas_rxtp.max_sdu = q_fetch(dsc,QF_fw_max_sdu);
+	qos->rxtp.max_sdu = q_fetch(dsc,QF_fw_max_sdu);
     if (q_present(dsc,QG_bhli)) {
 	sap->sas_addr.bhli.hl_type = q_fetch(dsc,QF_hli_type)+1;
 	switch (sap->sas_addr.bhli.hl_type) {
