@@ -1,6 +1,6 @@
 /* write.c - send AAL5 PDU */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -24,6 +24,7 @@ static void usage(const char *name)
 int main(int argc,char **argv)
 {
     struct sockaddr_atmpvc addr;
+    struct atm_qos qos;
     int s,size,offset;
 
     if (argc != 3 && argc != 4) usage(argv[0]);
@@ -34,9 +35,14 @@ int main(int argc,char **argv)
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
 	usage(argv[0]);
-    addr.sap_txtp.class = ATM_UBR;
-    addr.sap_txtp.max_sdu = strlen(argv[2])-offset;
     offset = argc == 3 ? 0 : atoi(argv[3]);
+    memset(&qos,0,sizeof(qos));
+    qos.txtp.class = ATM_UBR;
+    qos.txtp.max_sdu = strlen(argv[2])-offset;
+    if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
+	perror("setsockopt SO_ATMQOS");
+	return 1;
+    }
     if (connect(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("connect");
 	return 1;
