@@ -1,6 +1,6 @@
 /* br.c - block read */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -27,6 +27,7 @@ static void usage(const char *name)
 int main(int argc,char **argv)
 {
     struct sockaddr_atmpvc addr;
+    struct atm_qos qos;
     int s;
 
     if (argc != 2) usage(argv[0]);
@@ -37,8 +38,14 @@ int main(int argc,char **argv)
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
 	usage(argv[0]);
-    addr.sap_rxtp.class = ATM_UBR;
-    addr.sap_rxtp.max_sdu = BSIZE;
+    memset(&qos,0,sizeof(qos));
+    qos.rxtp.class = ATM_UBR;
+    qos.rxtp.max_sdu = BSIZE;
+    if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
+	perror("setsockopt SO_ATMQOS");
+	return 1;
+    }
+
     if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
 	return 1;
